@@ -8,8 +8,9 @@ ROOT.gROOT.SetBatch(True)
 ROOT.PyConfig.IgnoreCommandLineOptions = True
 """
 module_path = pathlib.Path(__file__).parent.absolute()
-macro_path = module_path
+macro_path = os.path.join(module_path, 'macros')
 stylesheet_path = os.path.join(module_path, 'stylesheets')
+resource_path = os.path.join(module_path, 'resources')
 
 # ROOT.gInterpreter.AddIncludePath(os.path.join(macro_path, "macros"))
 
