@@ -47,13 +47,12 @@ class AnalysisPathManager(PathManager):
         "benchmark_significance"                   : ("summary", "benchmark_significance.json")
     }
     
-    def __init__(self, study_name:str="", base_path:Optional[str]=None,
+    def __init__(self, study_name:Optional[str]=None, base_path:Optional[str]=None,
                  directories:Optional[Dict[str, str]]=None,
                  files:Optional[Dict[str, Union[str, Tuple[Optional[str], str]]]]=None):
         
         directories = combine_dict(self.DEFAULT_DIRECTORIES, directories)
         files       = combine_dict(self.DEFAULT_FILES, files)
-            
         super().__init__(base_path=base_path, directories=directories, files=files)
         
         self.study_name    = study_name
@@ -62,9 +61,15 @@ class AnalysisPathManager(PathManager):
         
     def update(self):
         if self.raw_base_path is None:
-            base_path = self.study_name
+            if self.study_name is None:
+                base_path = None
+            else:
+                base_path = self.study_name
         else:
-            base_path = os.path.join(self.raw_base_path, self.study_name)
+            if self.study_name is None:
+                base_path = self.raw_base_path
+            else:
+                base_path = os.path.join(self.raw_base_path, self.study_name)
         self.base_path = base_path
         
     def set_study_name(self, study_name:str):
