@@ -9,6 +9,7 @@ import numpy as np
 
 from quickstats import (semistaticmethod, DescriptiveEnum, DataclassObject,
                         ConfigComponent, ConfigParser, ConfigurableObject)
+from quickstats.utils.common_utils import combine_dict
 from .analysis_base import AnalysisBase
 from .data_preprocessing import fix_negative_weights, shuffle_arrays, get_class_label_encodings
 
@@ -96,7 +97,7 @@ class DataTransformer(object):
         if sample_weight_scales is None:
             self.sample_weight_scales = {}
         else:
-            self.sample_weight_scales = sample_weight_scales
+            self.sample_weight_scales = combine_dict(sample_weight_scales)
 
 class SplitAlgoBase(ConfigurableObject):
     def get_analysis_dataset_names(self) -> List[str]:
@@ -452,7 +453,23 @@ class DataLoader(AnalysisBase):
         self.set_data_transformer()
         self.set_split_algo()
         self.data_format = self.get_analysis_data_format()
+        self.weight_variable = self.config['names'].get("weight", None)
+        sample_weight_scales = self.config.get("rescale_weights", {}).get("data_loading", None)
+        self.set_weight_scales(sample_scales=sample_weight_scales)
         
+    def set_weight_scales(self, sample_scales:Optional[Dict]=None,
+                          lumi_scale:Optional[float]=None):
+        if (sample_scales is not None):
+            self.sample_weight_scales = combine_dict(sample_scales)
+        else:
+            self.sample_weight_scales = None
+        self.lumi_weight_scale = lumi_scale
+        if self.lumi_weight_scale is not None:
+            self.stdout.info(f'INFO: Set luminosity weight scale to {self.lumi_weight_scale}')
+        if self.sample_weight_scales is not None:
+            for sample, scale_factor in self.sample_weight_scales.items():
+                self.stdout.info(f'INFO: Set event weight scale for the sample "{sample}" to {scale_factor}')
+
     def set_data_transformer(self, data_transformer:Optional[DataTransformer]=None):
         if data_transformer is None:
             train_config = self.config["training"]
@@ -548,6 +565,16 @@ class DataLoader(AnalysisBase):
                                         fmt=self.data_format)
         self.stdout.info(f'INFO: Loading inputs for the sample "{sample}" from "{sample_path}"')
         df = self._read_data_from_file(sample_path, key=key, columns=columns, selection=selection)
+        if self.lumi_weight_scale is not None:
+            if self.weight_variable is None:
+                raise RuntimeError("cannot apply luminosity weight scale: weight variable not set")
+            df[self.weight_variable] = df[self.weight_variable] * self.lumi_weight_scale
+        if self.sample_weight_scales is not None:
+            if self.weight_variable is None:
+                raise RuntimeError("cannot apply sample weight scale: weight variable not set")
+            sample_weight_scale = self.sample_weight_scales.get(sample, None)
+            if sample_weight_scale is not None:
+                df[self.weight_variable] = df[self.weight_variable] * sample_weight_scale
         return df
     
     def get_sample_dfs(self, samples:List[str], key:Optional[str]=None,
@@ -705,7 +732,7 @@ class DataLoader(AnalysisBase):
             "samples"                : self.resolve_samples(channel_config['train_samples']),
             "train_variables"        : self.resolve_variables(channel_config['train_variables']),
             "class_label_map"        : class_label_map,
-            "weight_variable"        : self.config['names'].get("weight", None),
+            "weight_variable"        : self.weight_variable,
             "selection"              : channel_config.get('selection', None),
             "selection_variables"    : channel_config.get('selection_variables', None),
             "kinematic_region"       : channel_config.get('kinematic_region', None),
