@@ -84,7 +84,6 @@ class EventCategorization(DataLoader):
                          outdir=outdir,
                          verbosity=verbosity,
                          **kwargs)
-        
         # append the study name to the output directory
         self.set_study_name(study_name)
         self.do_blind  = do_blind
@@ -622,7 +621,8 @@ class EventCategorization(DataLoader):
                 weight_components.append(sample_selection)
             bs_datasets = bs_config.get("datasets", [])
             if bs_datasets and (not ignore_dataset_weight):
-                if (not bs_config["adaptive_dataset"]) or (sample in train_samples):
+                if ((not self.is_data_sample(sample)) and
+                    ((not bs_config["adaptive_dataset"]) or (sample in train_samples))):
                     weight_expr = self.get_analysis_dataset_weight_expr(bs_datasets)
                     weight_components.append(weight_expr)
         else:
@@ -634,7 +634,8 @@ class EventCategorization(DataLoader):
                 weight_components.append(sample_selection)
             eval_datasets = eval_config.get("datasets", [])
             if eval_datasets:
-                if (not eval_config["adaptive_dataset"]) or (sample in train_samples):
+                if ((not self.is_data_sample(sample)) and
+                    ((not eval_config["adaptive_dataset"]) or (sample in train_samples))):
                     weight_expr = self.get_analysis_dataset_weight_expr(eval_datasets)
                     weight_components.append(weight_expr)
             # remove data events in the signal region in case of blind analysis
@@ -1327,6 +1328,8 @@ class EventCategorization(DataLoader):
             benchmark_data['combined'] = combined_Z
     
     def run_channel_categorization(self, channel:str, cache_boundaries:bool=True,
+                                   cache_categories:bool=False,
+                                   cache_scores:bool=False,
                                    event_indices:Optional[Dict[str, np.ndarray]]=None):
         """
         Perform event categorization for a given channel.
@@ -1335,8 +1338,12 @@ class EventCategorization(DataLoader):
         ----------
         channel: str
            Name of analysis channel.
-        cache: bool, default = True
-           Cache categorization result.
+        cache boundaries: bool, default = True
+           Cache score boundaries (skipping boundary scan step).
+        cache categories: bool, default = True
+           Cache application of category columns (skipping apply category step).
+        cache scores: bool, default = True
+           Cache application of score columns (skipping apply score step).
         cat_event_number: (optional) dict
            If specified, event categorization will be performed on a specific set of events for each sample
            given in the dictionary. The format is { sample: array of event numbers }.
@@ -1344,9 +1351,10 @@ class EventCategorization(DataLoader):
         if event_indices is not None:
             self.stdout.info("INFO: Categorization will be done on a given set of event numbers for each sample")
         self.set_active_channel(channel)
-        self.load_channel_df(event_indices=event_indices)
+        self.load_channel_df(event_indices=event_indices, apply_score=not cache_scores)
         self.scan_bounds(cache=cache_boundaries)
-        self.apply_categories()
+        if not cache_categories:
+            self.apply_categories()
         self.evaluate_benchmark_significance()
     
     def save_dataframe_as_array(self, df:pd.DataFrame, savepath:str, fmt:Optional[str]=None):
@@ -1664,7 +1672,7 @@ class EventCategorization(DataLoader):
     
     def save_combined_yields(self, cache:bool=True):
         self.validate(check_channels=True)
-        category_names = self.get_all_category_names()
+        category_names = self.get_all_category_names(valid_only=False)
         yield_info = self.get_yield_files_and_description()
         for yield_file, description in yield_info.items():
             merged_yield_file = f"merged_{yield_file}"
