@@ -86,8 +86,8 @@ class NTupleConversionTool(AnalysisBase):
             # does not support chunksize
             from quickstats.utils.data_conversion import root2dataframe
             df = root2dataframe(filename, treename, columns=columns,
-                                mode=2,
-                                downcast=downcast, library="root")
+                                mode=2, downcast=downcast, library="root",
+                                multithread=False)
             df_iter = [df]
         elif library == "root_pandas":
             import root_pandas
