@@ -3,7 +3,6 @@ import json
 import click
 
 class DelimitedStr(click.Option):
-
     def type_cast_value(self, ctx, value):
         try:
             return [i.strip() for i in value.split(",")]
@@ -456,9 +455,13 @@ def generate_standard_asimov(**kwargs):
     Generate standard Asimov dataset
     """
     from quickstats.components import AsimovGenerator
+    from quickstats.utils.string_utils import split_str
     outname = kwargs.pop('outname')
     asimov_types = kwargs.pop('asimov_types')
-    asimov_types = [int(t) for t in asimov_types.split(',')]
+    try:
+        asimov_types = split_str(asimov_types, sep=",", cast=int)
+    except:
+        asimov_types = split_str(asimov_types, sep=",")
     fix_param = kwargs.pop('fix_param')
     profile_param = kwargs.pop('profile_param')
     snapshot_name = kwargs.pop('snapshot_name')
@@ -472,10 +475,11 @@ def generate_standard_asimov(**kwargs):
         'profile_param': profile_param,
         'snapshot_name': snapshot_name
     }
+    from quickstats.utils.string_utils import split_str
     if asimov_names is not None:
-        asimov_names = asimov_names.split(",")
+        asimov_names = split_str(asimov_names, sep=",")
     if asimov_snapshots is not None:
-        asimov_snapshots = asimov_snapshots.split(",")        
+        asimov_snapshots = split_str(asimov_snapshots, sep=",")
     generator = AsimovGenerator(**kwargs, config=config, verbosity=verbosity)
     generator.generate_standard_asimov(asimov_types, poi_scale=poi_scale,
                                        asimov_names=asimov_names,
