@@ -32,9 +32,10 @@ def inspect_ws(input_file, ws_name=None, data_name=None, mc_name=None, output_fi
     from quickstats.components import ExtendedModel
     model = ExtendedModel(input_file, ws_name=ws_name, mc_name=mc_name, data_name=data_name,
                           verbosity="WARNING")
+    from quickstats.utils.string_utils import split_str
     #items = items.split(",") if items is not None else None
-    include_patterns = include_patterns.split(",") if include_patterns is not None else None
-    exclude_patterns = exclude_patterns.split(",") if exclude_patterns is not None else None
+    include_patterns = split_str(include_patterns, ',') if include_patterns is not None else None
+    exclude_patterns = split_str(exclude_patterns, ',') if exclude_patterns is not None else None
     model.stdout.verbosity = "INFO"
     model.print_summary(items=items, save_as=output_file, detailed=detailed,
                         include_patterns=include_patterns, exclude_patterns=exclude_patterns)
\ No newline at end of file
