@@ -88,7 +88,8 @@ def likelihood_fit(**kwargs):
     """
     do_minos = kwargs.pop("minos")
     rebuild = kwargs.pop("rebuild")
-    pois = [poi for poi in kwargs.pop("pois").split(",") if poi]
+    from quickstats.utils.string_utils import split_str
+    pois = split_str(kwargs.pop("pois"), sep=',', remove_empty=True)
     _kwargs = {}
     for arg_name in ["outname", "save_log", "display", "save_result",
                      "export_as_np_pulls", "outdir", "save_ws", "save_snapshot"]:
