@@ -19,8 +19,9 @@ def process_rfile(filename, config_path, multithread, tag, flag, verbosity):
     Process a ROOT file based on RDataFrame routines.
     """
     from quickstats.components.processors import RooProcessor
+    from quickstats.utils.string_utils import split_str
     if flag is not None:
-        flags = [s.strip() for s in flag.split(",")]
+        flags = split_str(flag, sep=',', remove_empty=True)
     else:
         flags = []
     rprocessor = RooProcessor(config_path, multithread=multithread, flags=flags, verbosity=verbosity)
