@@ -48,7 +48,7 @@ def build_xml_ws(**kwargs):
     """
     Build workspace from XML config files
     """
-    version == kwargs.pop("version")
+    version = kwargs.pop("version")
     _kwargs = {}
     for arg_name in ["source", "use_binned", "data_storage_type",
                      "basedir", "verbosity"]:
@@ -224,10 +224,12 @@ def combine_ws(**kwargs):
     infile = kwargs.pop("input_workspace")
     outfile = kwargs.pop("output_workspace")
     import_class_code = kwargs.pop('import_class_code')
+    from quickstats.utils.string_utils import split_str
     if infile is not None:
         infiles = {}
-        for item in infile.split(","):
-            subitems = [i.strip() for i in item.split("=")]
+        items = split_str(infile, sep=",", remove_empty=True)
+        for item in items:
+            subitems = split_str(item, sep='=')
             if len(subitems) != 2:
                 raise ValueError("invalid format for the argument \"input_workspace\"")
             infiles[subitems[0]] = subitems[1]
