@@ -33,8 +33,17 @@ class AnalysisObject(AbstractObject):
         else:
             self.minimizer_cls = minimizer_cls
         if filename is not None:
-            self.setup_model(filename, data_name, binned_likelihood, ws_name, mc_name, 
-                             snapshot_name, fix_cache, fix_multi, verbosity=verbosity)
+            model_options = {
+                "fname"             : filename,
+                "ws_name"           : ws_name,
+                "mc_name"           : mc_name,
+                "data_name"         : data_name,
+                "snapshot_name"     : snapshot_name,
+                "binned_likelihood" : binned_likelihood,
+                "fix_cache"         : fix_cache,
+                "fix_multi"         : fix_multi
+            }
+            self.setup_model(**model_options)
             self.model._load_floating_auxiliary_variables()
             self.save_snapshot(self.kInitialSnapshotName, WSArgument.MUTABLE)
             self.set_poi(poi_name)
@@ -127,15 +136,10 @@ class AnalysisObject(AbstractObject):
         else:
             self.poi = self.model.get_poi(poi_name)
             if poi_name is not None:
-                self.stdout.info('INFO: POI set to "{}"'.format(poi_name))            
+                self.stdout.info(f'INFO: POI set to "{poi_name}"')
         
-    def setup_model(self, filename, data_name='combData', binned_likelihood:bool=True, 
-                    ws_name:Optional[str]=None, mc_name:Optional[str]=None, 
-                    snapshot_name:Optional[str]=None, fix_cache:bool=True, 
-                    fix_multi:bool=True, verbosity:Optional[Union[int, str]]="INFO"):
-        model = ExtendedModel(fname=filename, ws_name=ws_name, mc_name=mc_name, data_name=data_name, 
-                              snapshot_name=snapshot_name, binned_likelihood=binned_likelihood,
-                              fix_cache=fix_cache, fix_multi=fix_multi, verbosity=verbosity)
+    def setup_model(self, **kwargs):
+        model = ExtendedModel(**kwargs, verbosity=self.stdout.verbosity)
         self.model = model
     
     def setup_parameters(self, fix_param:str='', profile_param:str='', update_snapshot:bool=True):
@@ -186,23 +190,14 @@ class AnalysisObject(AbstractObject):
                                     f"POI used in this study. This parameter will still be floated during "
                                     f"unconditional likelihood fit / limit setting.", "red")
             
-    def setup_minimizer(self, minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
-                        precision:float=0.001, eps:float=1.0, retry:int=1, strategy:int=0, 
-                        num_cpu:int=1, offset:bool=True, optimize:bool=True, eigen:bool=False,
-                        hesse:bool=False, improve:int=0, max_calls:int=-1, max_iters:int=-1,
-                        print_level:int=-1, timer:bool=False, constrain_nuis:bool=True,
-                        batch_mode:bool=False, int_bin_precision:float=-1., minimizer_offset:int=1,
-                        verbosity:Optional[Union[int, str]]="INFO"):
+    def setup_minimizer(self, constrain_nuis:bool=True, **kwargs):
         
         minimizer = self.minimizer_cls("Minimizer", self.model.pdf, self.model.data,
-                                       workspace=self.model.workspace, verbosity=verbosity)
+                                       workspace=self.model.workspace,
+                                       verbosity=self.stdout.verbosity)
+        
+        nll_options = {k:v for k,v in kwargs.items() if k in ExtendedMinimizer._DEFAULT_NLL_OPTION_}
         
-        nll_options = {
-            "num_cpu": num_cpu,
-            "offset": offset,
-            "batch_mode": batch_mode,
-            "int_bin_precision": int_bin_precision
-        }
         if constrain_nuis:
             nll_options['constrain'] = self.model.nuisance_parameters
             nll_options['global_observables'] = self.model.global_observables
@@ -211,26 +206,10 @@ class AnalysisObject(AbstractObject):
                 nll_options['conditional_observables'] = conditional_obs
             #nll_commands.append(ROOT.RooFit.ExternalConstraints(ROOT.RooArgSet()))
         
-        minimizer_options = {
-            'minimizer_type'   : minimizer_type,
-            'minimizer_algo'   : minimizer_algo,
-            'default_strategy' : strategy,
-            'opt_const'        : optimize,
-            'precision'        : precision,
-            'eps'              : eps,
-            'retry'            : retry,
-            'eigen'            : eigen,
-            'hesse'            : hesse,
-            'minimizer_offset' : minimizer_offset,
-            'improve'          : improve,
-            'max_calls'        : max_calls,
-            'max_iters'        : max_iters,
-            'print_level'      : print_level,
-            'timer'            : timer
-        }
+        minimizer_options = {k:v for k,v in kwargs.items() if k in ExtendedMinimizer._DEFAULT_MINIMIZER_OPTION_}
 
-        minimizer.configure(**minimizer_options)
         minimizer.configure_nll(**nll_options)
+        minimizer.configure(**minimizer_options)
         self.minimizer = minimizer
         self.default_nll_options = nll_options
         self.default_minimizer_options = minimizer_options
@@ -238,7 +217,7 @@ class AnalysisObject(AbstractObject):
     def set_data(self, data_name:str='combData'):
         data = self.model.workspace.data(data_name)
         if not data:
-            raise RuntimeError(f"workspace does not contain the dataset \"{data_name}\"")
+            raise RuntimeError(f'workspace does not contain the dataset "{data_name}"')
         self.minimizer.set_data(data)
         self.model._data = data
         
@@ -259,10 +238,13 @@ class AnalysisObject(AbstractObject):
         if snapshot_name is not None:
             snapshot = self.model.workspace.getSnapshot(snapshot_name)
             if (not snapshot) and ('0x(nil)' in snapshot.__repr__()):
-                self.stdout.warning(f"WARNING: Snapshot \"{snapshot_name}\" does not exist.")
+                self.stdout.warning(f'WARNING: Snapshot "{snapshot_name}" does not exist.')
             else:
                 self.model.workspace.loadSnapshot(snapshot_name)
-                self.stdout.debug(f"DEBUG: Loaded snapshot \"{snapshot_name}\"")
+                self.stdout.debug(f'DEBUG: Loaded snapshot "{snapshot_name}"')
+                
+    def restore_snapshot(self):
+        self.load_snapshot(self.kCurrentSnapshotName)
     
     def get_variables(self, variable_type:Union[str, WSArgument], sort:bool=True):
         return self.model.get_variables(variable_type, sort=sort)
@@ -276,8 +258,18 @@ class AnalysisObject(AbstractObject):
                 if isinstance(variables, (str, WSArgument)):
                     variables = self.get_variables(variables)
                 self.model.workspace.saveSnapshot(snapshot_name, variables)
-            self.stdout.debug(f"DEBUG: Saved snapshot \"{snapshot_name}\"")
+            self.stdout.debug(f'DEBUG: Saved snapshot "{snapshot_name}"')
         
-    def save(self, fname:str, recreate:bool=True):
-        self.model.save(fname, recreate=recreate)
-        self.stdout.info(f"INFO: Saved workspace file as `{fname}`")
\ No newline at end of file
+    def save(self, filename:str, recreate:bool=True, rebuild:bool=True):
+        self.model.save(filename, recreate=recreate, rebuild=rebuild)
+        self.stdout.info(f'INFO: Saved workspace file as "{filename}"')
+        
+        
+    def decompose_nll(self, fmt:str="pandas"):
+        from quickstats.utils.roofit_utils import decompose_nll
+        if not self.minimizer.nll:
+            self.minimizer.create_nll()
+            result = decompose_nll(self.minimizer.nll, self.model.global_observables, fmt=fmt)
+            self.minimizer.nll = None
+            return result
+        return decompose_nll(self.minimizer.nll, self.model.global_observables, fmt=fmt)
\ No newline at end of file
