@@ -215,12 +215,13 @@ class AsymptoticCLs(AnalysisObject):
         qmu = 2*(nll_value - nll_muhat)
         if qmu < 0 :
             if self.last_fit_status < 0:
-                raise RuntimError("detected fit failure causing a negative qmu value;  "
+                raise RuntimeError("detected fit failure causing a negative qmu value;  "
                                   "try to increase the number of retries upon a failed fit "
                                   "(the option --retry in CLI) or check that your model can "
                                   "accurately represent the dataset of interest")
             else:
-                raise RuntimeError("qmu is negative which means the globally minimized "
+                raise RuntimeError(f"qmu is negative (uncond NLL: {nll_muhat}, cond NLL: {nll_value}) "
+                                   "which means the globally minimized "
                                    "NLL is larger then the NLL of interest; this can happen "
                                    "when the true global minimum is outside the valid range "
                                    "of the parameter of interest; try to prevent automatic "
