@@ -23,6 +23,7 @@ from quickstats.utils.root_utils import load_macro
 from quickstats.utils.common_utils import str_list_filter
 from quickstats.utils.roofit_utils import (get_variable_attributes, variable_collection_to_dataframe,
                                            get_relevant_components)
+from quickstats.utils.string_utils import split_str
 from quickstats.interface.root import TH1, RooDataSet, RooArgSet, RooMsgService
 from quickstats.interface.root import roofit_extension as rf_ext
 from quickstats.interface.root.roofit_extension import get_str_data
@@ -130,7 +131,7 @@ class ExtendedModel(AbstractObject):
         if val is None:
             self._initial_snapshots = []
         elif isinstance(val, str):
-            self._initial_snapshots = [s.strip() for s in val.split(',') if s]
+            self._initial_snapshots = split_str(val, sep=',', remove_empty=True)
         elif isinstance(val, list):
             self._initial_snapshots = val
         else:
@@ -468,7 +469,7 @@ class ExtendedModel(AbstractObject):
             elif isinstance(param_expr, dict):
                 param_dict = param_dict
             else:
-                raise ValueError('invalid format for parameter expression: {}'.format(param_expr))
+                raise ValueError(f'invalid format for parameter expression: {param_expr}')
         elif param_expr is None:
         # if param_expr is None, all parameters will be parsed as None by default
             param_dict = {param.GetName():None for param in source}
@@ -480,7 +481,7 @@ class ExtendedModel(AbstractObject):
         Example: "param_1,param_2=0.5,param_3=-1,param_4=1,param_5=_0_100,param_6=__100,param_7=_0_"
         '''
         param_str = param_str.replace(' ', '')
-        param_list = param_str.split(',')
+        param_list = split_str(param_str, sep=',', remove_empty=True)
         param_dict = {}
         for param_expr in param_list:
             expr = param_expr.split('=')
@@ -490,12 +491,12 @@ class ExtendedModel(AbstractObject):
             # case both parameter name and value is given
             elif len(expr) == 2:
                 param_name = expr[0]
-                param_values = [float(v) if v else None for v in expr[1].split("_")]
+                param_values = [float(v) if v else None for v in split_str(expr[1], sep="_")]
                 if len(param_values) == 1:
                     param_values = param_values[0]
                 param_dict[param_name] = param_values
             else:
-                raise ValueError('invalid parameter expression: {}'.format(param))
+                raise ValueError(f'invalid parameter expression: {param}')
         return param_dict
     
     @staticmethod
@@ -545,7 +546,7 @@ class ExtendedModel(AbstractObject):
         return unconstrained_nuis
     
     def set_constrained_nuisance_parameters_to_nominal(self, set_constant:bool=False):
-        constrain_pdf, constrained_nuis, _ = self.pair_constraints(fmt="arglist", sort=False)
+        constrain_pdf, constrained_nuis, _ = self.pair_constraints(fmt="argset", sort=False)
         for pdf, nuis in zip(constrain_pdf, constrained_nuis):
             pdf_class = pdf.ClassName()
             if pdf_class in ["RooGaussian", "RooBifurGauss"]:
@@ -553,7 +554,7 @@ class ExtendedModel(AbstractObject):
             elif pdf_class == "RooPoisson":
                 nuis.setVal(1)
             else:
-                raise RuntimeError(f"constraint term \"{pdf.GetName()}\" has unsupported type \"{pdf_class}\"")
+                raise RuntimeError(f'constraint term "{pdf.GetName()}" has unsupported type "{pdf_class}"')
         if set_constant:
             for nuis in constrained_nuis:
                 nuis.setConstant(1)
@@ -627,7 +628,7 @@ class ExtendedModel(AbstractObject):
                             if not found_sigma:
                                 self.stdout.info(f'INFO: Sigma for pdf {constr_name} not found. Uisng 1.0.')
                             else:
-                                self.stdout.info('INFO: Uisng {} for sigma of pdf {}'.format(old_sigma_value, constr_name))
+                                self.stdout.info(f'INFO: Uisng {old_sigma_value} for sigma of pdf {constr_name}')
 
                             prefit_variation = old_sigma_value
                         elif constraint.IsA() == ROOT.RooPoisson.Class():
