@@ -15,7 +15,7 @@ import quickstats
 from quickstats import semistaticmethod, Timer
 from quickstats.components import ROOTObject
 from quickstats.components.modelling import TreeDataSource, PdfFitTool
-from quickstats.utils.common_utils import combine_dict, execute_multi_tasks
+from quickstats.utils.common_utils import combine_dict, execute_multi_tasks, in_notebook
 
 class DataModelling(ROOTObject):
     
@@ -361,7 +361,9 @@ class DataModelling(ROOTObject):
         import matplotlib.pyplot as plt
         if save_as is not None:
             plt.savefig(save_as, bbox_inches="tight")
-        plt.show()
+        if in_notebook():
+            plt.show()
+        return ax
 
     def run(self, filename:str, tree_name:str, save_summary_as:Optional[str]=None, 
             save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None):
