@@ -14,7 +14,7 @@ from typing import List, Optional
 import numpy as np
 
 from quickstats.components import ExtendedModel, ExtendedMinimizer
-from quickstats.utils import root_utils, common_utils
+from quickstats.utils import root_utils, common_utils, string_utils
 
 import ROOT
 
@@ -294,7 +294,7 @@ class NuisanceParameterPull(object):
         # include all NPs
         else:
             nuis_list = [np.GetName() for np in model.get_variables("nuisance_parameter")]
-        include_patterns = parameter.split(',')
+        include_patterns = string_utils.split_str(parameter, sep=',', remove_empty=True)
         if parameter:
             nuis_to_include = []
             for nuis_name in nuis_list:
@@ -303,7 +303,7 @@ class NuisanceParameterPull(object):
                     nuis_to_include.append(nuis_name)
         else:
             nuis_to_include = nuis_list
-        exclude_patterns = exclude.split(',')
+        exclude_patterns =  string_utils.split_str(exclude, sep=',', remove_empty=True)
         nuis_to_exclude = []
         if exclude_patterns:
             for nuis_name in nuis_list:
