@@ -5,6 +5,7 @@ import ROOT
 from quickstats.components import AnalysisBase
 from quickstats.maths.numerics import is_float
 from quickstats.utils.common_utils import format_delimiter_enclosed_text
+from quickstats.utils.string_utils import split_str, remove_whitespace
 
 from .settings import (GEN_ASIMOV_ACTION, FLOAT_ASIMOV_ACTION, RAW_ASIMOV_ACTION,
                        FIT_ASIMOV_ACTION, RESET_ASIMOV_ACTION, FIX_SYST_ASIMOV_ACTION,
@@ -30,14 +31,14 @@ class AsimovHandler(AnalysisBase):
         self.saved_snapshots = [RAW_ASIMOV_ACTION]
         
     def parse_setup(self, setup_str:str):
-        setup_str = setup_str.replace(" ", "")
+        setup_str = remove_whitespace(setup_str)
         fix_setup = []
         profile_setup = []
         invalid_setup = []
         undefined_setup = []
-        tokens = setup_str.split(",")
+        tokens = split_str(setup_str, sep=',', remove_empty=True)
         for token in tokens:
-            subtokens = token.split("=")
+            subtokens = split_str(token, sep='=')
             if len(subtokens) != 2:
                 invalid_setup.append(token)
                 continue
@@ -46,7 +47,7 @@ class AsimovHandler(AnalysisBase):
             if not self.model.workspace.var(name):
                 undefined_setup.append(name)
                 continue
-            expr_tokens = expr.split("_")
+            expr_tokens = split_str(expr, sep='_')
             if len(expr_tokens) not in [1, 3]:
                 invalid_setup.append(token)
                 continue
