@@ -19,11 +19,12 @@ from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.basics import WSArgument
 from quickstats.interface.root.roofit_extension import get_str_data
 
-kUnique        = 0b00001
-kRedefined     = 0b00010
-kReconstituted = 0b00100
-kRenamed       = 0b01000
-kIdentical     = 0b10000
+kUnique        = 0b000001
+kRedefined     = 0b000010
+kReconstituted = 0b000100
+kRenamed       = 0b001000
+kIdentical     = 0b010000
+kRegrouped     = 0b100000
 
 class WSItemType(GeneralEnum):
     
@@ -100,7 +101,8 @@ class ComparisonData:
         self.df_2 = pd.DataFrame({"name":[], "class":[], "definition":[]})
         self.df_merge = None
         self.df_factorized = None
-        self.possible_mappings = None
+        self.mapping_statistics_left = None
+        self.mapping_statistics_right = None
 
     def set_title(self, title:str):
         self.title = title
@@ -143,54 +145,42 @@ class ComparisonData:
         a = target_df['args_left'].values
         b = target_df['args_right'].values
         names = target_df['name'].values
-        results = {}
-        statistics = {}
+        # left to right mapping
+        statistics_left = {}
+        # right to left mapping
+        statistics_right = {}
         size = len(a)
         for i in range(size):
-            a_i = a[i]
-            b_i = b[i]
             name_i = names[i]
-            size_i = len(a_i)
-            result_i = {}
-            count_i = {}
+            size_i = len(a[i])
             for j in range(size_i):
                 a_ij = a[i][j]
                 b_ij = b[i][j]
                 if a_ij != b_ij:
-                    if a_ij not in count_i:
-                        count_i[a_ij] = 0
-                    result_i[a_ij] = b_ij
-                    count_i[a_ij] += 1
-            for a_ij, count in count_i.items():
-                if count != 1:
-                    result_i.pop(a_ij)
-                else:
-                    if a_ij not in statistics:
-                        statistics[a_ij] = {"candidates":[], "counts": {"total": 0}}
-                    b_ij = result_i[a_ij]
-                    if b_ij not in statistics[a_ij]["candidates"]:
-                        statistics[a_ij]["candidates"].append(b_ij)
-                        statistics[a_ij]["counts"][b_ij] = 0
-                    statistics[a_ij]["counts"][b_ij] += 1
-                    statistics[a_ij]["counts"]["total"] += 1
-            results[name_i] = result_i
-        self.possible_mappings = results
-        self.mapping_statistics = statistics
+                    if a_ij not in statistics_left:
+                        statistics_left[a_ij] = {}
+                    if b_ij not in statistics_right:
+                        statistics_right[b_ij] = {}
+                    if b_ij not in statistics_left[a_ij]:
+                        statistics_left[a_ij][b_ij] = 0
+                    if a_ij not in statistics_right[b_ij]:
+                        statistics_right[b_ij][a_ij] = 0
+                    statistics_left[a_ij][b_ij] += 1
+                    statistics_right[b_ij][a_ij] += 1
+        self.mapping_statistics_left = statistics_left
+        self.mapping_statistics_right = statistics_right
     
     @staticmethod
     def _get_combined_mapping_statistics(all_mappings:List[Dict]):
         combined_mapping = {}
         for mapping in all_mappings:
-            for key, value in mapping.items():
+            for key, candidate_counts in mapping.items():
                 if key not in combined_mapping:
-                    combined_mapping[key] = {"candidates":[], "counts":{"total":0}}
-                for candidate in value["candidates"]:
-                    if candidate not in combined_mapping[key]["candidates"]:
-                        combined_mapping[key]["candidates"].append(candidate)
-                for candidate, count in value["counts"].items():
-                    if candidate not in combined_mapping[key]["counts"]:
-                        combined_mapping[key]["counts"][candidate] = 0
-                    combined_mapping[key]["counts"][candidate] += count
+                    combined_mapping[key] = {}
+                for candidate, count in candidate_counts.items():
+                    if candidate not in combined_mapping[key]:
+                        combined_mapping[key][candidate] = 0
+                    combined_mapping[key][candidate] += count
         return combined_mapping
     
     def process_mappings(self, mappings:Dict):
@@ -198,7 +188,7 @@ class ComparisonData:
             self.process()
         df = self.df_merge
         df.set_index("name", inplace=True)
-        for name_1, name_2 in mappings.items():
+        for name_1, name_2 in mappings['one_to_one'].items():
             if name_1 in df.index:
                 series_1 = df.loc[name_1]
                 if series_1['_merge'] != "left_only":
@@ -225,14 +215,43 @@ class ComparisonData:
                     # object mapped to object of another type
                     df.loc[name_1, ["name_alt"]] = name_2
                     df.loc[name_1, ["category"]] = "remapped"
-                    
-            if name_2 in df.index:
-                series_2 = df.loc[name_2]
-                if series_2['_merge'] != "right_only":
+            # object of another type mapped to object of current type
+            elif name_2 in df.index:
+                if df.loc[name_2]['_merge'] != "right_only":
                     continue
                 # object mapped to object of another type
                 df.loc[name_2, ["name_alt"]] = name_1
                 df.loc[name_2, ["category"]] = "remapped"
+
+        for name_1, names_2 in mappings['split'].items():
+            if name_1 in df.index:
+                if df.loc[name_1]['_merge'] != "left_only":
+                    continue
+                df.loc[name_1, ["name_alt"]] = "::".join(names_2)
+                df.loc[name_1, ["category"]] = "splitted"
+            for name_2 in names_2:
+                # object type did not change
+                if name_2 not in df.index:
+                    continue
+                if df.loc[name_2]['_merge'] != "right_only":
+                    continue
+                df.loc[name_2, ["name_alt"]] = name_1
+                df.loc[name_2, ["category"]] = "splitted"
+
+        for name_2, names_1 in mappings['merge'].items():
+            for name_1 in names_1:
+                if name_1 not in df.index:
+                    continue
+                if df.loc[name_1]['_merge'] != "left_only":
+                    continue
+                df.loc[name_1, ["name_alt"]] = name_2
+                df.loc[name_1, ["category"]] = "merged"
+            if name_2 in df.index:
+                if df.loc[name_2]['_merge'] != "right_only":
+                    continue
+                df.loc[name_2, ["name_alt"]] = "::".join(names_1)
+                df.loc[name_2, ["category"]] = "merged"
+                    
         df.reset_index(inplace=True)
         
     def process(self):
@@ -247,7 +266,7 @@ class ComparisonData:
         expression_right = df_merge['definition_right'].str.replace(regex, "")
         df_merge['same_expression'] = expression_left == expression_right
         df_merge['category'] = df_merge['_merge']
-        new_categories = ["identical", "redefined", "reconstituted", "renamed", "remapped"]
+        new_categories = ["identical", "redefined", "reconstituted", "renamed", "remapped", "splitted", "merged"]
         df_merge['category'] = df_merge['category'].cat.add_categories(new_categories)
         mask_identical = (df_merge['_merge'] == "both") & (df_merge['same_definition'])
         mask_redefined = (df_merge['_merge'] == "both") & (~df_merge['same_definition'])
@@ -255,7 +274,6 @@ class ComparisonData:
         df_merge.loc[mask_redefined, ['category']] = 'redefined'
         df_merge['name_alt'] = None
         self.df_merge = df_merge
-        
         if self.is_composite:
             self.factorize_definition()
             self.evaluate_possible_mappings()
@@ -276,6 +294,8 @@ class ComparisonData:
                     unique_right  = different name, found in right only, no common server with right                   
                     renamed       = different name, same definition, same servers
                     remapped      = different name, different definition, same servers
+                    splitted      = a single instance of left plays the role of multiple instances of right
+                    merged        = roles of multiple instances of left played by a single instance of right
                     common        = same name
                     distinct      = different_name
                     left          = everything found in left
@@ -296,12 +316,16 @@ class ComparisonData:
                     df = df[~df['same_expression']]
                 if category == "reconstituted":
                     df = df[df['same_expression']]
-        elif category in ["distinct", "renamed", "remapped"]:
+        elif category in ["distinct", "renamed", "remapped", "splitted", "merged"]:
             df = self.df_merge[self.df_merge['_merge'] != "both"]
             if category == "renamed":
                 df = df[df['category'] == "renamed"]
             elif category == "remapped":
                 df = df[df['category'] == "remapped"]
+            elif category == "splitted":
+                df = df[df['category'] == "splitted"]
+            elif category == "merged":
+                df = df[df['category'] == "merged"]                
         elif category == "left_only":
             df = self.df_merge[self.df_merge['_merge'] == "left_only"]
         elif category == "right_only":
@@ -317,7 +341,7 @@ class ComparisonData:
         else:
             raise ValueError('invalid category (choose from "identical", "redefined", "reconstituted", '
                              '"reexpressed", "left_only", "right_only", "unique_left", "unique_right", '
-                             '"renamed", "remapped", "common", "distinct", "any")')
+                             '"renamed", "remapped", "splitted", "merged", "common", "distinct", "any")')
         # make a copy and reset index
         df = df.reset_index(drop=True)
         return df
@@ -394,27 +418,37 @@ class ComparisonData:
         df_2.set_index('name', inplace=True)
         contents_1 = []
         contents_2 = []
-        for name, alt_name in mapping_1.items():
+        # cases:
+        #[{"LA": "RA1"}, {"LA": "RB"}, {"LA" : "RB1::RB2::RB3"}, {"LA": "RA1::RA2::RA3"}, {"LA": "RB1::RA1"}
+        # {"RA": "LA1"}, {"RA": "LB"}, {"RA": "LA1::LA2::LA3"}, {"RA": "LB1::LB2::LB3"}, {"RA": "LB1::LA1"}]
+        for name, alt_names in mapping_1.items():
             contents_1.append(df_1.loc[name]["definition_left"])
-            # left same item type -> right same item type
-            if (alt_name in mapping_2):
-                if (mapping_2[alt_name] == name):
-                    contents_2.append(df_2.loc[alt_name]["definition_right"])
-            # left same item type -> right diff item type
-            elif (alt_name  in ext_ref):
-                contents_2.append(ext_ref[alt_name])
-            else:
-                contents_2.append(f"{alt_name} [definition not inferred]")
-        for name, alt_name in mapping_2.items():
+            content_components = []
+            alt_names = alt_names.split("::")
+            for alt_name in alt_names:
+                # left same item type -> right same item type;
+                if (alt_name in mapping_2):
+                    content_components.append(df_2.loc[alt_name]["definition_right"])
+                # left same item type -> right diff item type;
+                elif (alt_name  in ext_ref):
+                    content_components.append(ext_ref[alt_name])
+                else:
+                    content_components.append(f"{alt_name} [definition not inferred]")
+            contents_2.append(" & ".join(content_components))
+        for name, alt_names in mapping_2.items():
             # left same item type -> right same item type; already dealt with
-            if (alt_name in mapping_1):
+            if ("::" not in alt_names) and (alt_names in mapping_1):
                 continue
             contents_2.append(df_2.loc[name]["definition_right"])
-            # left diff item type -> right same item type
-            if (alt_name  in ext_ref):
-                contents_1.append(ext_ref[alt_name])
-            else:
-                contents_1.append(f"{alt_name} [definition not inferred]")            
+            content_components = []
+            alt_names = alt_names.split("::")
+            for alt_name in alt_names:
+                # left diff item type -> right same item type; 
+                if (alt_name  in ext_ref):
+                    content_components.append(ext_ref[alt_name])
+                else:
+                    content_components.append(f"{alt_name} [definition not inferred]")
+            contents_1.append(" & ".join(content_components))
         return contents_1, contents_2
     
     def get_summary_str(self, visibility:int=0b00011, indent:str="   ",
@@ -423,11 +457,12 @@ class ComparisonData:
             Arguments:
                 visibility: integer
                     boolean mask for showing definitions of certain objects
-                    0b00001 = show definitions for unique objects
-                    0b00010 = show definitions for redefined objects
-                    0b00100 = show definitions for reconstituted objects
-                    0b01000 = show definitions for renamed/remapped objects
-                    0b10000 = show definitions for identical objects
+                    0b000001 = show definitions for unique objects
+                    0b000010 = show definitions for redefined objects
+                    0b000100 = show definitions for reconstituted objects
+                    0b001000 = show definitions for renamed/remapped objects
+                    0b010000 = show definitions for identical objects
+                    0b010000 = show definitions for regroupped objects
         """
         if ext_ref is None:
             ext_ref = {}
@@ -497,11 +532,35 @@ class ComparisonData:
             content_kwargs = {
                 "contents_1": contents_1,
                 "contents_2": contents_2,
-                "title": "Remapped Object (Modified Name, Modified Definition)",
+                "title": "Remapped Object (Modified Name, Modified Definition, One to One)",
                 "title_color": "bright yellow",
                 "indent": indent,
                 "show_content": visibility & kRenamed
             }
+            summary_str += self._get_mapped_content_summary_str(**content_kwargs)
+            
+            df = self.get_df("splitted")
+            contents_1, contents_2 = self._get_ext_mapping_contents(df, ext_ref)
+            content_kwargs = {
+                "contents_1": contents_1,
+                "contents_2": contents_2,
+                "title": "Splitted Object (Modified Name, Modified Definition, One to Many)",
+                "title_color": "bright yellow",
+                "indent": indent,
+                "show_content": visibility & kRegrouped
+            }
+            summary_str += self._get_mapped_content_summary_str(**content_kwargs)
+            
+            df = self.get_df("merged")
+            contents_1, contents_2 = self._get_ext_mapping_contents(df, ext_ref)
+            content_kwargs = {
+                "contents_1": contents_1,
+                "contents_2": contents_2,
+                "title": "Merged Object (Modified Name, Modified Definition, Many to One)",
+                "title_color": "bright yellow",
+                "indent": indent,
+                "show_content": visibility & kRegrouped
+            }
             summary_str += self._get_mapped_content_summary_str(**content_kwargs)            
             
             df = self.get_df("unique_left")
@@ -914,11 +973,41 @@ class WSComparer(AbstractObject):
             self.stdout.info(f"INFO: All requested data have been successfully loaded. Time taken: {t.interval:.3f}s.")
             return self._get_combined_mapping_statistics()
         
-        mapping_statistics_pdf = self.data[pdf_item_name].mapping_statistics
-        mapping_statistics_function = self.data[function_item_name].mapping_statistics
-        combined_mapping = ComparisonData._get_combined_mapping_statistics([mapping_statistics_pdf,
-                                                                            mapping_statistics_function])
-        return combined_mapping
+        pdf_mapping_statistics_left = self.data[pdf_item_name].mapping_statistics_left
+        function_mapping_statistics_left = self.data[function_item_name].mapping_statistics_left
+        pdf_mapping_statistics_right = self.data[pdf_item_name].mapping_statistics_right
+        function_mapping_statistics_right = self.data[function_item_name].mapping_statistics_right        
+        combined_mapping_left = ComparisonData._get_combined_mapping_statistics([pdf_mapping_statistics_left,
+                                                                                 function_mapping_statistics_left])
+        combined_mapping_right = ComparisonData._get_combined_mapping_statistics([pdf_mapping_statistics_right,
+                                                                                  function_mapping_statistics_right])        
+        return combined_mapping_left, combined_mapping_right
+    
+    def _get_mappings(self, mapping_statistics_left:Dict, mapping_statistics_right:Dict):
+        mapping_left = {k:list(v) for k,v in mapping_statistics_left.items()}
+        mapping_right = {k:list(v) for k,v in mapping_statistics_right.items()}
+        mappings = {
+            "one_to_one": {},
+            "split": {},
+            "merge": {}
+        }
+        for key, candidates in mapping_left.items():
+            if len(candidates) == 1:
+                candidate = candidates[0]
+                inverse_candidates = mapping_right.get(candidate, [])
+                if len(inverse_candidates) == 1 and inverse_candidates[0] == key:
+                    mappings['one_to_one'][key] = candidate
+                elif len(inverse_candidates) > 1:
+                    # surjection for left to right mapping
+                    if not all(len(mapping_left.get(icandidate, [])) == 1 for icandidate in inverse_candidates):
+                        continue
+                    mappings['merge'][candidate] = inverse_candidates
+            else:
+                # surjection for right to left mapping
+                if not all(mapping_right.get(candidate, [None])[0] == key for candidate in candidates):
+                    continue
+                mappings['split'][key] = candidates
+        return mappings
     
     def _get_items_with_definition(self):
         relevant_items = []
@@ -935,17 +1024,11 @@ class WSComparer(AbstractObject):
         if len(relevant_items) > 0:
             self.stdout.info("INFO: Processing renamed and remapped variables...")
             with Timer() as t:
-                combined_mapping_statistics = self._get_combined_mapping_statistics()
-                valid_mappings = {}
-                for name_1 in combined_mapping_statistics:
-                    statistics = combined_mapping_statistics[name_1]
-                    candidates = statistics["candidates"]
-                    if len(candidates) == 1:
-                        name_2 = candidates[0]
-                        valid_mappings[name_1] = name_2
+                map_stat_left, map_stat_right = self._get_combined_mapping_statistics()
+                mappings = self._get_mappings(map_stat_left, map_stat_right)
                 for item_str in relevant_items:
                     data = self.data[item_str]
-                    data.process_mappings(valid_mappings)
+                    data.process_mappings(mappings)
             self.stdout.info(f"INFO: Processing finished. Time taken: {t.interval:.3f}s")
  
     def process_data(self):
