@@ -12,8 +12,9 @@ from quickstats import semistaticmethod, AbstractObject, GeneralEnum, PathManage
 from quickstats.utils.io import Verbosity
 from quickstats.utils.xml_tools import TXMLTree
 from quickstats.maths.numerics import to_bool
-from quickstats.utils.root_utils import load_macro, get_macro_TClass
+from quickstats.utils.root_utils import load_macro, get_macro_TClass, get_macro_dir
 from quickstats.utils.common_utils import set_unlimited_stacksize
+from quickstats.utils.string_utils import split_str
 
 import ROOT
 
@@ -49,9 +50,9 @@ ConversionOperators = {
     XMLAttribType.BOOL: to_bool,
     XMLAttribType.INT: int,
     XMLAttribType.FLOAT: float,
-    XMLAttribType.STR_LIST  : lambda s: [i.strip() for i in s.split(",") if i],
-    XMLAttribType.INT_LIST  : lambda s: [int(i) for i in s.split(",") if i],
-    XMLAttribType.FLOAT_LIST: lambda s: [float(i) for i in s.split(",") if i]
+    XMLAttribType.STR_LIST  : lambda s: split_str(s, sep=',', remove_empty=True),
+    XMLAttribType.INT_LIST  : lambda s: split_str(s, sep=',', remove_empty=True, cast=int),
+    XMLAttribType.FLOAT_LIST: lambda s: split_str(s, sep=',', remove_empty=True, cast=float)
 }
 
 class XMLWSBase(AbstractObject):
