@@ -1093,7 +1093,11 @@ class XMLWSBuilder(XMLWSBase):
                                                         required=not is_counting, default="")
         syst_domains = self._translate_node_attrib(node, "ImportSyst", sample.process,
                                                    required=False, default=SELF_SYST_DOMAIN_KEYWORD)
-        syst_domains = re.sub('\s+', '', syst_domains).split(",")
+        syst_domains = re.sub('\s+', '', syst_domains)
+        if not syst_domains:
+            syst_domains = [SELF_SYST_DOMAIN_KEYWORD]
+        else:
+            syst_domains = syst_domains.split(",")
         sample.set_syst_domains(syst_domains)
         
         # assemble the yield central value
