@@ -20,6 +20,7 @@ import quickstats
 from quickstats import semistaticmethod, AbstractObject, GeneralEnum, Timer
 from quickstats.components import ExtendedModel
 from quickstats.utils.io import Verbosity
+from quickstats.utils.xml_tools import TXMLTree
 from quickstats.utils.root_utils import load_macro, get_macro_dir, is_corrupt
 from quickstats.utils.common_utils import format_delimiter_enclosed_text
 from quickstats.maths.numerics import is_float, pretty_value
@@ -78,7 +79,7 @@ class XMLWSCombiner(XMLWSBase):
         self.reset_comb_ws()
  
     def parse_config_xml(self, filename:str) -> None:
-        root = self._xml_to_dict(filename)
+        root = TXMLTree.load_as_dict(filename)
         combination_config = {
             'output_file'      : self._get_node_attrib(root, "OutputFile", required=True),
             'workspace_name'   : self._get_node_attrib(root, "WorkspaceName", required=False,
@@ -245,7 +246,9 @@ class XMLWSCombiner(XMLWSBase):
             'model_config_name' : self._get_node_attrib(node, "ModelConfigName", required=False),
             'data_name'         : self._get_node_attrib(node, "DataName", required=False, default="combData"),
             'simplified_import' : self._get_node_attrib(node, "SimplifiedImport", required=False,
-                                                        default="false", dtype="bool")
+                                                        default="false", dtype="bool"),
+            'argset_list'       : [],
+            'argset_strict'     : False
         }
         channel = channel_config["channel_name"]
         if channel in self.channel_config:
@@ -265,6 +268,8 @@ class XMLWSCombiner(XMLWSBase):
                 self.read_channel_rename_node(channel, child_node)
             elif tag == "POIList":
                 self.read_channel_poi_node(channel, child_node)
+            elif tag == "ArgSetList":
+                self.read_channel_argset_node(channel, child_node)
             else:
                 raise RuntimeError(f"unknown item `{tag}`")
         self.stdout.info(f"INFO: Successfully validated XML input for the channel \"{channel}\".")
@@ -272,7 +277,7 @@ class XMLWSCombiner(XMLWSBase):
     def read_channel_rename_node(self, channel:str, node:Dict) -> None:
         ext_file = self._get_node_attrib(node, "InputFile", required=False)
         if ext_file is not None:
-            node = self._xml_to_dict(ext_file)
+            node = TXMLTree.load_as_dict(ext_file)
         child_nodes = node['children']
         for child_node in child_nodes:
             tag = child_node['tag']
@@ -281,6 +286,14 @@ class XMLWSCombiner(XMLWSBase):
             else:
                 raise RuntimeError(f"unknown channel item `{tag}`")
                 
+    def read_channel_argset_node(self, channel:str, node:Dict) -> None:
+        argset_list = self._get_node_attrib(node, "Input", required=False,
+                                            default=[], dtype="str_list")
+        argset_strict = self._get_node_attrib(node, "Strict", required=False,
+                                              default="True", dtype="bool")
+        self.channel_config[channel]['argset_list'] = argset_list
+        self.channel_config[channel]['argset_strict'] = argset_strict
+                
     def validate_channel_input(self, channel:str):
         self._channel_check(channel)
         self.stdout.info(f"INFO: Checking input workspace for the channel \"{channel}\".")
@@ -346,7 +359,19 @@ class XMLWSCombiner(XMLWSBase):
         # fix unspecified names
         channel_config['workspace_name'] = model.workspace.GetName()
         channel_config['model_config_name'] = model.model_config.GetName()
-        channel_config['data_name'] = model.data.GetName()            
+        channel_config['data_name'] = model.data.GetName()
+        
+        argset_list = channel_config["argset_list"]
+        for argset in argset_list:
+            argset_obj = model.workspace.obj(argset)
+            if not argset_obj:
+                if channel_config["argset_strict"]:
+                    raise RuntimeError(f'Argument set "{argset}" does not exist in the channel "{channel}"')
+                else:
+                    self.stdout.warning(f'WARNING: Argument set "{argset}" does not exist in the channel "{channel}". Skipped.')
+                    continue
+            if not isinstance(argset_obj, ROOT.RooArgSet):
+                raise RuntimeError(f'Argument set "{argset}" from the channel "{channel}" is an instance of RooArgSet object')
         self.stdout.info(f"INFO: Successfully validated input workspace for the channel \"{channel}\".")
         
     def read_rename_syst_node(self, channel:str, node:Dict) -> None:
@@ -620,8 +645,32 @@ class XMLWSCombiner(XMLWSBase):
         self.import_object(self.tmp_ws, data,
                            ROOT.RooFit.RenameVariable(old_cat_name, new_cat_name),
                            silent=False)
+        self.update_channel_argset(channel, workspace, self.tmp_ws)
         self.tmp_nuis.add(model.nuisance_parameters.snapshot(), True)
         self.tmp_glob.add(model.global_observables.snapshot(), True)
+        
+    def update_channel_argset(self, channel:str, orig_ws:ROOT.RooWorkspace, new_ws:ROOT.RooWorkspace):
+        channel_config = self.channel_config[channel]
+        argset_map = {}
+        for argset_name in channel_config["argset_list"]:
+            argset_old = orig_ws.obj(argset_name)
+            assert isinstance(argset_old, ROOT.RooArgSet)
+            components = []
+            for arg in argset_old:
+                names = [arg.GetName(), f"{arg.GetName()}_{channel}"]
+                for name in names:
+                    obj = new_ws.obj(name)
+                    if obj:
+                        components.append(name)
+                        break
+                else:
+                    raise RuntimeError(f'unable to locate the variable "{arg.GetName()}" in the renamed workspace '
+                                       f'for the channel {channel}')
+            new_argset_name = f"{argset_name}_{channel}"
+            if (new_argset_name in self.argsets) or (new_argset_name in argset_map):
+                raise RuntimeError(f'duplicated ArgSet object "{new_argset_name}"')
+            argset_map[new_argset_name] = components
+        self.argsets.update(argset_map)
     
     def rename(self, save_rename_ws:bool=True) -> None:
         title = format_delimiter_enclosed_text(f"Rename PDFs, functions and variables",
@@ -706,11 +755,23 @@ class XMLWSCombiner(XMLWSBase):
         self.comb_ws.defineSet(self.KEYWORDS["comb_glob_name"], self.tmp_glob, True)
         self.stdout.info(f'INFO: Defining variable set "{self.KEYWORDS["comb_obs_name"]}"')
         self.comb_ws.defineSet(self.KEYWORDS["comb_obs_name"], self.comb_obs, True)
-        
+        self.define_argsets(self.comb_ws)    
         if save_combine_ws:
             filename = self._get_comb_ws_path()
             self.stdout.info(f'INFO: Saving temporary workspace to "{filename}"')
             self.comb_ws.writeToFile(filename, True)
+            
+    def define_argsets(self, ws:ROOT.RooWorkspace):
+        for argset_name, arg_names in self.argsets.items():
+            argset = ROOT.RooArgSet()
+            for arg_name in arg_names:
+                arg = ws.obj(arg_name)
+                if not arg:
+                    raise RuntimeError(f'failed to locate object "{arg_name}" from the combined workspace when '
+                                       f'defining the ArgSet "{argset_name}"')
+                argset.add(arg)
+            self.stdout.info(f'INFO: Defining argument set "{argset_name}"')
+            ws.Import(argset, argset_name, True)
         
     def get_combined_model_config(self) -> None:
         pois = ROOT.RooArgSet()
@@ -727,7 +788,7 @@ class XMLWSCombiner(XMLWSBase):
                     poi.setConstant(True)
                 else:
                     poi.setRange(poi_min, poi_max)
-                    if (poi_value > poi_min) and (poi_value < poi_max):
+                    if (poi_value is not None) and ((poi_value > poi_min) and (poi_value < poi_max)):
                         poi.setVal(poi_value)
                 pois.add(poi)
             else:
@@ -785,7 +846,8 @@ class XMLWSCombiner(XMLWSBase):
                               minimizer_config=self.minimizer_config,
                               title_indent_str="")    
         
-    def finalize(self, load_combine_ws:bool=True, save_final_ws:bool=True) -> None:
+    def finalize(self, load_combine_ws:bool=True, save_final_ws:bool=True,
+                 import_class_code:bool=True) -> None:
         if load_combine_ws:
             self.load_cache_ws("combine")
         
@@ -793,7 +855,8 @@ class XMLWSCombiner(XMLWSBase):
         self.stdout.info(f'INFO: Creating ModelConfig "{self.combination_config["model_config_name"]}"')
         model_config = self.get_combined_model_config()
         self.comb_ws.Import(model_config)
-        self.comb_ws.importClassCode()
+        if import_class_code:
+            self.import_class_code(self.comb_ws)
         
         title = format_delimiter_enclosed_text(f"Generate Asimov Dataset",
                                                delimiter="-", indent_str="")
@@ -813,12 +876,16 @@ class XMLWSCombiner(XMLWSBase):
             filename = self.combination_config["output_file"]
             self.stdout.info(f'INFO: Saving final combined workspace to "{filename}"')
             self.comb_ws.writeToFile(filename, True)
+            
+    def reset_combination(self):
+        self.argsets = {}
     
     def create_combined_workspace(self, infiles:Optional[Dict[str, str]]=None,
                                   outfile:Optional[str]=None,
                                   save_rename_ws:bool=False,
                                   save_combine_ws:bool=False,
-                                  save_final_ws:bool=True) -> None:
+                                  save_final_ws:bool=True,
+                                  import_class_code:bool=True) -> None:
         with Timer() as t:
             # override path to the input channel workspaces
             if infiles is not None:
@@ -826,6 +893,7 @@ class XMLWSCombiner(XMLWSBase):
                     if channel not in infiles:
                         raise ValueError(f'missing input workspace path for the channel "{channel}"')
                     self.channel_config[channel]['input_file'] = infiles[channel]
+            self.reset_combination()
             # override path to the combined workspace
             if outfile is not None:
                 self.combination_config['output_file'] = outfile
@@ -834,5 +902,6 @@ class XMLWSCombiner(XMLWSBase):
             self.print_input_summary()
             self.rename(save_rename_ws=save_rename_ws)
             self.combine(load_rename_ws=save_rename_ws, save_combine_ws=save_combine_ws)
-            self.finalize(load_combine_ws=save_combine_ws, save_final_ws=save_final_ws)
+            self.finalize(load_combine_ws=save_combine_ws, save_final_ws=save_final_ws,
+                          import_class_code=import_class_code)
         self.stdout.info(f"INFO: Total time taken: {t.interval:.3f}s")
\ No newline at end of file
