@@ -727,7 +727,11 @@ class XMLWSModifier(XMLWSBase):
             obj_name, obj_type = self._get_object_name_and_type_from_expr(expr)
             arg = old_ws.arg(obj_name)
             if not arg:
-                raise RuntimeError(f"object {obj_name} does not exist in the original workspace")
+                if self.config['strict']:
+                    raise RuntimeError(f"object {obj_name} does not exist in the original workspace")
+                else:
+                    self.stdout.warning(f"WARNING: object {obj_name} does not exist in the original workspace")
+                    continue
             if obj_type == WSObjectType.VARIABLE:
                 result = re.search(r"\[(.+)\]", expr)
                 if not result:
