@@ -1,3 +1,4 @@
+from typing import Any
 from enum import Enum
 
 class GeneralEnum(Enum):
@@ -17,18 +18,38 @@ class GeneralEnum(Enum):
             return expr
         elif isinstance(expr, int):
             return cls(expr)
-        _expr = expr.strip().upper()
-        if _expr in cls.__members__:
-            return cls[_expr]
-        elif _expr in cls.__aliases__:
-            return cls(cls.__aliases__[_expr])
+        _expr = expr.strip().lower()
+        members = cls.get_members_map()
+        if _expr in members:
+            return cls(members[_expr])
         else:
-            cls.on_parse_exception(expr)
+            # check aliases
+            aliases = cls.get_aliases_map()
+            if _expr in aliases:
+                return cls(cls.parse(aliases[_expr]))
+        cls.on_parse_exception(expr)
             
     @classmethod
     def get_members(cls):
         return [i.lower() for i in cls.__members__]
     
+    @classmethod
+    def get_members_map(cls):
+        return {k.lower():v for k, v in cls.__members__.items()}
+    
+    @classmethod
+    def get_aliases_map(cls):
+        return {k.lower():v for k, v in cls.__aliases__.items()}
+    
+    @classmethod
+    def has_member(cls, name:str):
+        return name in cls.__members__
+    
+    @classmethod
+    def get_member_by_attribute(cls, attribute:str, value:Any):
+        members = cls.__members__
+        return next((x for x in members.values() if getattr(x, attribute) == value), None)
+    
 class DescriptiveEnum(GeneralEnum):
     
     def __new__(cls, value:int, description:str=""):
