@@ -5,6 +5,8 @@ from quickstats.interface.root.macros import load_macros, load_macro
 from quickstats.interface.root.TArrayData import TArrayData
 from quickstats.interface.root.TH1 import TH1
 from quickstats.interface.root.TH2 import TH2
+from quickstats.interface.root.TF1 import TF1
+from quickstats.interface.root.TFile import TFile
 from quickstats.interface.root.RooRealVar import RooRealVar
 from quickstats.interface.root.RooAbsData import RooAbsData
 from quickstats.interface.root.RooDataSet import RooDataSet
