@@ -1,4 +1,5 @@
 from typing import Union, Any, List, Dict, Optional
+from fractions import Fraction
 
 import numpy as np
 
@@ -27,7 +28,7 @@ def to_string(val:Any, precision:int=8) -> str:
     return str(val)
 
 def to_rounded_float(val:Any, precision:int=8) -> float:
-    return round(float(val), precision)
+    return round(float(Fraction(val)), precision)
 
 def pretty_value(val:Union[int, float], precision:int=8)->Union[int, float]:
     if isinstance(val, float):
