@@ -211,4 +211,8 @@ class AbstractPlot:
         if ymax is not None:
             ylim[1] = ymax                
         ax.set_xlim(*xlim)
-        ax.set_ylim(*ylim)
\ No newline at end of file
+        ax.set_ylim(*ylim)
+    
+    @staticmethod
+    def close_all_figures():
+        matplotlib.pyplot.close()
\ No newline at end of file
