@@ -58,7 +58,13 @@ QUICKSTATS_PALETTES  = dict(
     darklines=["#000000", "#0019f5", "#377d22", "#ea3423", "#f2a93b", "#7f170e"],
     default=['#000000', '#F2385A', '#4AD9D9', '#FDC536', '#125125', '#E88EED', '#B68D40'],
     hist_series=['#99CCFF', '#EFCF6E', '#64CB34'],
-    shading_light=["#FBF0D9", "C0E7DC", "E4C0F4", "B3D4E8"],
-    shading_medium=["#E0DCC5", "C5E00CC", "E0C5D4", "C5C5E0"],
-    shading_dark=["#C1B98A", "8AC199", "C18AA9", "8B8AC1"]
+    shading_light=["#FBF0D9", "#C0E7DC", "#E4C0F4", "#B3D4E8"],
+    shading_medium=["#E0DCC5", "#C5E00C", "#E0C5D4", "#C5C5E0"],
+    shading_dark=["#C1B98A", "#8AC199", "#C18AA9", "#8B8AC1"],
+    ibm=["#648fff", "#785ef0", "#dc267f", "#fe6100", "#ffb000"],
+    butterflycore=["#f4cccc", "#ffe599", "#b6d7a8", "#9fc5e8", "#9C98DB", "#D6BFAB", "#876659"],
+    simple_contrast=["#ED553B", "#20639B", "#3CAEA3", "#F6D55C", "#BF51A0", "#735245"],
+    checker=["#068EAC", "#DE5B7D", "#4A9451", "#F97623", "#FAE215", "#BF51A0",
+             "#84E4D0", "#B32A32", "#ADE85B", "#FFA41A", "#505BB6", "#596D3D",
+             "#2C388E", "#5B3F7B", "#8D89C2", "#6586B3", "#CCA18D", "#735245"]
 )
\ No newline at end of file
