@@ -12,6 +12,8 @@ import collections.abc
 
 import numpy as np
 
+from .string_utils import split_str
+
 def timely_info(green_text, normal_text, adjust=40):
     print(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), '\033[92m[INFO]\033[0m', '\033[92m{}\033[0m'.format(green_text).rjust(40, ' '), normal_text)
 
@@ -240,7 +242,7 @@ def filter_by_wildcards(targets:List, conditions:Optional[Union[str, List]]=None
         return targets
     selected = []
     if isinstance(conditions, str):
-        conditions = [i.strip() for i in conditions.split(",")]
+        conditions = split_str(conditions, sep=',')
     for target in targets:
         if any([fnmatch.fnmatch(target, condition) for condition in conditions]):
             selected.append(target)
