@@ -18,7 +18,7 @@ root_datatypes = ["bool", "Bool_t", "Byte_t", "char", "char*", "Char_t",
                   "unsigned long", "unsigned long long",
                   "unsigned short", "UShort_t"]
 
-uproot_datatypes = ["double", "float", "int", "int64_t", "char*", "int32_t", "uint64_t"]
+uproot_datatypes = ["double", "float", "int", "int64_t", "char*", "int32_t", "uint64_t", "uint32_t"]
 
 class ConversionMode(DescriptiveEnum):
     ALL = (0, "Convert all variable types")
@@ -129,8 +129,8 @@ def get_rdf_column_type(rdf, column_name:str):
 
 def reduce_vector_types(column_types:List[str]):
     reduced_column_types = []
-    vec_expr1 = re.compile(r"ROOT::VecOps::RVec<(\w+)>")
-    vec_expr2 = re.compile(r"vector<(\w+)>")
+    vec_expr1 = re.compile(r"ROOT::VecOps::RVec<([ \w]+)>")
+    vec_expr2 = re.compile(r"vector<([ \w]+)>")
     def match(pattern, string):
         result = pattern.match(string)
         if result:
@@ -169,6 +169,7 @@ def rdf2numpy(rdf, columns:Union[Dict[str, str], List[str]]=None,
     else:
         save_columns = list(columns)
     conversion_mode = ConversionMode.parse(mode)
+
     if conversion_mode in [ConversionMode.REMOVE_NON_STANDARD_TYPE,
                            ConversionMode.REMOVE_NON_ARRAY_TYPE]:
         column_types = np.array([get_rdf_column_type(rdf, column_name) for column_name in save_columns])
@@ -227,7 +228,7 @@ def root2numpy(filename:Union[str, List[str]], treename:str,
                columns:Union[Dict[str, str], List[str]]=None,
                cut:Optional[str]=None, convert_vectors:bool=True,
                mode:Union[str, int, ConversionMode]=1,
-               library:str="auto"):
+               library:str="auto", multithread:bool=True):
     if isinstance(filename, str) and os.path.isdir(filename):
         filename = glob.glob(os.path.join(filename, "*.root"))
     conversion_mode = ConversionMode.parse(mode)
@@ -236,11 +237,13 @@ def root2numpy(filename:Union[str, List[str]], treename:str,
     if library.lower() == "auto":
         library = get_default_library(custom_columns=isinstance(columns,dict))
     if library.lower() == "root":
-        import ROOT
-        rdf = ROOT.RDataFrame(treename, filename)
-        return rdf2numpy(rdf, columns=columns, cut=cut,
-                         convert_vectors=convert_vectors,
-                         mode=mode)
+        from quickstats.interface.root.helper import RMultithreadEnv
+        with RMultithreadEnv(multithread):
+            import ROOT
+            rdf = ROOT.RDataFrame(treename, filename)
+            return rdf2numpy(rdf, columns=columns, cut=cut,
+                             convert_vectors=convert_vectors,
+                             mode=mode)
     elif library.lower() == "uproot":
         if isinstance(columns, dict):
             raise RuntimeError('defining new columns are not supported when using "uproot" as the library')
@@ -282,7 +285,7 @@ def root2dataframe(filename:Union[str, List[str]], treename:str,
                    cut:Optional[str]=None,
                    mode:Union[str, int, ConversionMode]=1,
                    downcast:bool=True,
-                   library:str="auto"):
+                   library:str="auto", multithread:bool=True):
     conversion_mode = ConversionMode.parse(mode)
     if conversion_mode == ConversionMode.REMOVE_NON_ARRAY_TYPE:
         library = "root"    
@@ -292,7 +295,8 @@ def root2dataframe(filename:Union[str, List[str]], treename:str,
         numpy_data = root2numpy(filename, treename, columns=columns, cut=cut,
                                 convert_vectors=True,
                                 mode=mode,
-                                library=library)
+                                library=library,
+                                multithread=multithread)
         result = numpy2dataframe(numpy_data)
     elif library.lower() == "uproot":
         if isinstance(columns, dict):
