@@ -50,12 +50,14 @@ def format_comparison_text(text_left:str, text_right:str,
 
 @total_ordering
 class Verbosity(Enum):
+    
     CRITICAL = 50
     ERROR = 40
     WARNING = 30
     INFO = 20
     DEBUG = 10
     NOTSET = 0
+    
     def __lt__(self, other):
         if self.__class__ is other.__class__:
             return self.value < other.value
@@ -64,6 +66,15 @@ class Verbosity(Enum):
         elif isinstance(other, str):
             return self.value < getattr(self, other.upper()).value
         return NotImplemented
+    
+    def __eq__(self, other):
+        if self.__class__ is other.__class__:
+            return self.value == other.value
+        elif isinstance(other, int):
+            return self.value == other
+        elif isinstance(other, str):
+            return self.value == getattr(self, other.upper()).value
+        return NotImplemented
 
 class VerbosePrint:
     @property
