@@ -5,7 +5,7 @@ import fnmatch
 
 import ROOT
 
-from .string_utils import remove_whitespace
+from .string_utils import remove_whitespace, split_str
 
 def copy_attributes(source:"ROOT.RooAbsArg", target:"ROOT.RooAbsArg"):
     if source is target:
@@ -337,7 +337,7 @@ def recover_formula(formula:str):
     variables = variables_regex.findall(formula)
     if len(variables) != 1:
         raise RuntimeError("invalid RooFit factory expression")
-    variables = [i.strip() for i in variables[0].split(",") if i.strip()]
+    variables = split_str(variables[0], sep=',', remove_empty=True)
     recovered_formula = expression
     for i, variable in enumerate(variables):
         recovered_formula = re.sub(f"@{i}(?![0-9])", variable, recovered_formula)
