@@ -1,4 +1,4 @@
-from typing import Optional
+from typing import Optional, Callable
 import re
 
 whitespace_trans = str.maketrans('', '', " \t\r\n\v")
@@ -18,14 +18,17 @@ def split_lines(string:str, comment_string:Optional[str]="#", remove_blank:bool=
     return lines
 
 
-def split_str(string:str, strip:bool=True, remove_empty:bool=False):
+def split_str(string:str, sep:str=None, strip:bool=True,
+              remove_empty:bool=False, cast:Optional[Callable]=None):
     if strip:
-        tokens = [s.strip() for s in string.strip().split()]
+        tokens = [s.strip() for s in string.strip().split(sep)]
     else:
-        tokens = string.split()
+        tokens = string.split(sep)
     if remove_empty:
         # remove empty tokens
         tokens = [s for s in tokens if s]
+    if cast is not None:
+        tokens = [cast(s) for s in tokens]
     return tokens
 
 def remove_whitespace(string:str):
